/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.ArrayList;
import java.util.Date;

import de.bsvrz.sys.funclib.bitctrl.dua.schnittstellen.IObjektWeckerListener;

/**
 * Ein Ereignis das man mit dem Zeitsimulator einplanen kann
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class Ereignis implements IObjektWeckerListener {

	/**
	 * Eriegnistypen, die unterstuetzt sind
	 *
	 * @author bachraty
	 */
	public enum EreignissTyp {
		EREIGNIS_KEIN, EREIGNIS_SM_OFFLINE, EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN, EREIGNIS_SM_GEHT_ONLINE, EREIGNIS_DE_WIRD_ARCHIVIERT, EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT, EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN, EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN, EREIGNIS_DE_FLACKERT, EREIGNIS_DE_ANWORTET_NICHT_MEHR, EREIGNIS_KOSISTENZTEST, EREIGNIS_ABFRAGEMODUL_PARAM, EREIGNIS_SIMULATION_ENDE,
	};

	/**
	 * Typ des ereignises
	 */
	private EreignissTyp ereignis = EreignissTyp.EREIGNIS_KEIN;
	/**
	 * Objekt, der an dem das Ereignis passiert
	 */
	private final Object objekt;

	/**
	 * Standardkonstruktor
	 *
	 * @param objekt
	 *            Objekt
	 * @param ereignis
	 *            Eriegnis
	 */
	public Ereignis(final Object objekt, final EreignissTyp ereignis) {
		this.ereignis = ereignis;
		this.objekt = objekt;
	}

	@Override
	public void alarm() {
		String s = "";
		if (objekt instanceof DeSim) {
			s = ((DeSim) objekt).getDePid();
		} else if (objekt instanceof AbfragePufferDatenTester) {
			s = "AbfragePufferDatenTester";
		}

		System.out.println("## EREIGNIS Real @ "
				+ new Date(System.currentTimeMillis()) + " Simulation  @ "
				+ new Date(ZeitSimulator.getTimeMillis()) + " " + ereignis);

		if ((this.ereignis == EreignissTyp.EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN)
				&& (objekt instanceof DeSim)) {
			final DeSim de = (DeSim) objekt;
			de.goOnline(true);
		} else if (this.ereignis == EreignissTyp.EREIGNIS_SM_GEHT_ONLINE) {
			final ArrayList<DeSim> list = (ArrayList<DeSim>) objekt;
			for (final DeSim de : list) {
				de.goOnline(false);
			}
			// DeSim de = (DeSim)objekt;
			// de.goOnline(false);
		} else if (this.ereignis == EreignissTyp.EREIGNIS_SM_OFFLINE) {
			final ArrayList<DeSim> list = (ArrayList<DeSim>) objekt;
			for (final DeSim de : list) {
				de.goOffline();
			}

			// DeSim de = (DeSim)objekt;
			// de.goOffline();
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_WIRD_ARCHIVIERT)
				&& (objekt instanceof DeSim)) {
			final DeSim de = (DeSim) objekt;
			de.parametriereArchiv(true);
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT)
				&& (objekt instanceof DeSim)) {
			final DeSim de = (DeSim) objekt;
			de.parametriereArchiv(false);
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_KOSISTENZTEST)
				&& (objekt instanceof AbfragePufferDatenTester)) {
			final AbfragePufferDatenTester tester = (AbfragePufferDatenTester) objekt;
			tester.testKonsistenz();
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_ABFRAGEMODUL_PARAM)
				&& (objekt instanceof AbfragePufferDatenTester)) {
			final AbfragePufferDatenTester tester = (AbfragePufferDatenTester) objekt;
			tester.sendeAbfrageModulParametrierung();
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_SIMULATION_ENDE)
				&& (objekt instanceof AbfragePufferDatenTester)) {
			final AbfragePufferDatenTester tester = (AbfragePufferDatenTester) objekt;
			tester.ende();
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN)
				&& ((objekt instanceof LveDeSim) || (objekt instanceof AxlDeKanal255Sim))) {
			if (objekt instanceof LveDeSim) {
				final LveDeSim de = (LveDeSim) objekt;
				de.setAnfrageNichtVerstanden(true);
			} else {
				final AxlDeKanal255Sim de = (AxlDeKanal255Sim) objekt;
				de.setAnfrageNichtVerstanden(true);
			}

		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN)
				&& ((objekt instanceof LveDeSim) || (objekt instanceof AxlDeKanal255Sim))) {
			if (objekt instanceof LveDeSim) {
				final LveDeSim de = (LveDeSim) objekt;
				de.setAnfrageNichtVerstanden(false);
			} else {
				final AxlDeKanal255Sim de = (AxlDeKanal255Sim) objekt;
				de.setAnfrageNichtVerstanden(false);
			}
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_FLACKERT)
				&& (objekt instanceof DeSim)) {
			final DeSim de = (DeSim) objekt;
			de.flackern();
		} else if ((this.ereignis == EreignissTyp.EREIGNIS_DE_ANWORTET_NICHT_MEHR)
				&& (objekt instanceof DeSim)) {
			final DeSim de = (DeSim) objekt;
			de.goOfflineOhneMeldung();
		}
	}

	@Override
	public String toString() {
		final String s = this.objekt != null ? this.objekt.toString() + " "
				: "<<null>> ";

		if (this.ereignis == EreignissTyp.EREIGNIS_KEIN) {
			return s + "kein Ereignis";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_SM_OFFLINE) {
			return "SM geht offline";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_GEHT_ONLINE_MIT_DATEN) {
			return "SM geht online";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_SM_GEHT_ONLINE) {
			return "SM geht online";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_WIRD_ARCHIVIERT) {
			return s + "DE wird archiviert";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_WIRD_NICHT_ARCHIVIERT) {
			return s + "DE wird nicht archiviert";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_NICHT_VERSTEHEN) {
			return s + "Negative Quittung";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_KANN_ABFRAGE_WIEDER_VERSTEHEN) {
			return s + "negative Quittung ist weg???";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_FLACKERT) {
			return s + "DE meldet sich dauernd an und ab";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_DE_ANWORTET_NICHT_MEHR) {
			return s + "DE antwortet nicht";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_KOSISTENZTEST) {
			return s + "Konsistenztest";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_ABFRAGEMODUL_PARAM) {
			return s + "Abfragemodul wird umparametriert";
		}
		if (this.ereignis == EreignissTyp.EREIGNIS_SIMULATION_ENDE) {
			return s + "Ende der Simulation";
		}
		return "*** Fehler ***";
	}

}
