/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer;

import java.util.Collection;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;

/**
 * Einfache Klassse die ermoeglicht dass Mehrere SteuerModule in der selben zeit
 * simuliert werden, es gibt ur weiter alle Daten jedem Steuermodul, und auch
 * jede CallbackMethode (wie initialize) wird weitergeitet
 *
 * @author BitCtrl Systems GmbH, Bachraty
 *
 */
public class SmRunner implements StandardApplication {

	/**
	 * Liste aller SM
	 */
	private final Collection<SteuerModulSim> listSM = new LinkedList<>();
	/**
	 * Verbindung zum Datenverteiler
	 */
	private ClientDavInterface dav;
	/**
	 * Es gibt immer nur ein DeRunner in der Applikation, deswegen static
	 */
	private static SmRunner deRunner = new SmRunner();

	/**
	 * Ergibt eine Singleton Instanz
	 *
	 * @return Die Instanz
	 */
	static final SmRunner getInstance() {
		return SmRunner.deRunner;
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		// ShutdownHook wird beim beenden gerufen
		Runtime.getRuntime().addShutdownHook(new ShutdownHook());
		dav = connection;
		for (final SteuerModulSim sm : listSM) {
			sm.initialize(connection);
		}
		DeSim.sendeArchivParametrierung();
	}

	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		for (final SteuerModulSim sm : listSM) {
			final ArgumentList argList = new ArgumentList(argumentList.getArgumentStrings());
			sm.parseArguments(argList);
		}
	}

	/**
	 * Abmeldung vom DAV
	 */
	public void disconnect() {
		dav.disconnect(false, "");
	}

	/**
	 * Fuegt ein SM in die Liste ein
	 *
	 * @param sm
	 */
	public void add(final SteuerModulSim sm) {
		listSM.add(sm);
	}

	/**
	 * Damit bei Beendung der Applikation sich alle De's unmelden vom DAV
	 */
	protected class ShutdownHook extends Thread {
		@Override
		public void run() {
			try {
				for (final SteuerModulSim sm : listSM) {
					sm.unsubscribe();
				}
				disconnect();
			} catch (final Exception e) {
				System.out.println(e.getMessage());
				e.printStackTrace();
			}
			System.out.println("Quit");
		}
	}

	/**
	 * Weil das Terminate button in Eclipse ruft halt(), wird das ShutdownHook
	 * nicht gerufen deswegen wollen wir mit exit() beenden
	 */
	void run() {
		int x = 0;
		System.out.println("Druecke 'q' fuer beenden.");
		while (true) {
			try {
				x = System.in.read();
			} catch (final Exception e) {
				System.out.println(e.getMessage());
				Runtime.getRuntime().exit(1);
			}
			if (x == 'q') {
				Runtime.getRuntime().exit(0);
			}
		}
	}
}
