/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer.aggreg;

import org.junit.Assert;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dua.abfrpuffer.AbstraktAggregator;
import de.bsvrz.dua.abfrpuffer.AggregatorGeschwindigkeit;
import de.bsvrz.dua.abfrpuffer.Verbindung;

/**
 * Ermoeglicht die Klasse AggregatorGeschwindigkeit zu testen
 *
 * @author BitCtrl Systems GmbH, Bachraty
 */
public class GeschwindigkeitTester extends AggregatorGeschwindigkeit {

	/**
	 * Referenzwert mit dem die Ergebnisse vergleicht werden
	 */
	private Data referenzWert;

	/**
	 * Setzt dem Referenzwert
	 *
	 * @param d
	 *            Korrektes Ergebniss
	 */
	public void setReferenzWert(final Data d) {
		referenzWert = d;
	}

	/**
	 * Stat senden im Testfall werden wir mit Referenzwerten vergleichen
	 */
	@Override
	public void sendeDatenSatz(final SystemObject objekt,
			final ResultData resDaten) {

		final Data ergebniss = resDaten.getData();

		for (final String element : AbstraktAggregator.FZ_ATTRIB) {
			Assert.assertEquals(
					"Datum:\n" + resDaten + "\n\n" + referenzWert + "\n\n"
							+ element + ".Wert: ",
					referenzWert.getItem(element)
							.getUnscaledValue(AbstraktAggregator.STR_WERT)
							.doubleValue(), ergebniss.getItem(element)
							.getUnscaledValue(AbstraktAggregator.STR_WERT)
							.doubleValue(), 0);
			Assert.assertEquals(
					"Datum:\n" + resDaten + "\n\n" + referenzWert + "\n\n"
							+ element + ".Wert: ",
					referenzWert.getItem(element)
							.getItem(AbstraktAggregator.STR_GUETE)
							.getUnscaledValue(AbstraktAggregator.STR_INDEX)
							.doubleValue(),
					ergebniss.getItem(element)
							.getItem(AbstraktAggregator.STR_GUETE)
							.getUnscaledValue(AbstraktAggregator.STR_INDEX)
							.doubleValue(), 0);
			Assert.assertEquals(
					"Datum:\n" + resDaten + "\n\n" + referenzWert + "\n\n"
							+ element + ".Wert: ",
					referenzWert.getItem(element)
							.getItem(AbstraktAggregator.STR_GUETE)
							.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN)
							.longValue(),
					ergebniss.getItem(element)
							.getItem(AbstraktAggregator.STR_GUETE)
							.getUnscaledValue(AbstraktAggregator.STR_VERFAHREN)
							.longValue());
		}

		if (Verbindung.VERBOSE) {
			System.out.println("Ergebniss :" + resDaten.getData());
			System.out.println("Referenz  :" + referenzWert);
			System.out.println("---------");
		}
	}
}
