/*
 * Segment 4 Datenübernahme und Aufbereitung, SWE 4.5 Abfrage Puffer
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.dua.abfrpuffer.dtimport;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dua.abfrpuffer.AbfrPufferKonstanten;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAUtensilien;
import de.bsvrz.sys.funclib.bitctrl.dua.test.CSVImporter;

/**
 * Liest die Ausgangsdaten eines Fahrstreifens ein.
 *
 * @author BitCtrl Systems GmbH, Thierfelder
 *
 *         Modifiziet, Bachraty
 */

public class TestFSImporterGeschwKlasse extends CSVImporter {

	/**
	 * Verbindung zum Datenverteiler.
	 */
	private static ClientDavInterface dav;

	/**
	 * An dieser Spalte beginnen die wirklichen Daten.
	 */
	protected static final int OFFSET = 1;

	/**
	 * Anzahl der Spalten in CSV.
	 */
	protected static final int SPALTEN = 56;

	/**
	 * T.
	 */
	private static long intervall = AbfrPufferKonstanten.STUNDE_IN_MS;
	/**
	 * Zeitstempel des Datensatzes.
	 */
	private Date zeitStempel;
	/**
	 * Datumformat zum Parsen von Daten in String-Format.
	 */
	private final SimpleDateFormat df;

	/**
	 * Standardkonstruktor.
	 *
	 * @param dav
	 *            Datenverteier-Verbindung
	 * @param csvQuelle
	 *            Quelle der Daten (CSV-Datei)
	 * @throws Exception
	 *             falls dieses Objekt nicht vollständig initialisiert werden
	 *             konnte
	 */
	public TestFSImporterGeschwKlasse(final ClientDavInterface dav, final String csvQuelle) throws Exception {
		super(csvQuelle);
		if (TestFSImporterGeschwKlasse.dav == null) {
			TestFSImporterGeschwKlasse.dav = dav;
		}

		/**
		 * Tabellenkopf überspringen.
		 */
		this.getNaechsteZeile();

		zeitStempel = new Date();
		df = new SimpleDateFormat("dd.MM.yyyy HH:mm");
	}

	/**
	 * Setzt Datenintervall.
	 *
	 * @param t
	 *            Datenintervall in ms
	 */
	public static final void setT(final long t) {
		TestFSImporterGeschwKlasse.intervall = t;
	}

	/**
	 * Liefert den Zeitstempel.
	 *
	 * @return ZeitStemepel
	 */
	public final Date getZeitStempel() {
		return zeitStempel;
	}

	/**
	 * Erfragt die nächste Zeile innerhalb der CSV-Datei als einen Datensatz der
	 * übergebenen Attributgruppe.
	 *
	 * @param atg
	 *            eine Attributgruppe (KZD oder LZD)
	 * @param geschwDaten
	 *            definiert, ob es sich um Gescheindigkeitsdaten handelt
	 * @return ein Datensatz der übergebenen Attributgruppe mit den Daten der
	 *         nächsten Zeile oder <code>null</code>, wenn der Dateizeiger am
	 *         Ende ist
	 */
	public final Data getNaechstenDatensatz(final AttributeGroup atg, final boolean geschwDaten) {
		Data datensatz = TestFSImporterGeschwKlasse.dav.createData(atg);

		if (datensatz != null) {
			final String[] zeile = this.getNaechsteZeile();
			if (zeile != null) {
				try {

					final String[] praefix = new String[] { "q", "v", "s", "v85" };
					final String[] aName = new String[] { "Kfz", "PkwÄ", "KfzNk", "PkwG", "Pkw", "Krad", "Lfw", "LkwÄ",
							"PkwA", "Lkw", "Bus", "LkwK", "LkwA", "SattelKfz" };

					if (atg.getPid().equals(DUAKonstanten.ATG_LZD)) {
						datensatz = TestFSImporterGeschwKlasse.setLZDleer(datensatz, geschwDaten);
					}

					for (int i = 0; i < praefix.length; i++) {
						for (int j = 0; j < aName.length; j++) {
							final int wert = Integer
									.parseInt(zeile[(i * aName.length) + j + TestFSImporterGeschwKlasse.OFFSET]);
							TestFSImporterGeschwKlasse.setAttribut(praefix[i] + aName[j], wert, datensatz);
						}
					}

					datensatz.getTimeValue("T").setMillis(TestFSImporterGeschwKlasse.intervall);
					zeitStempel = df.parse(zeile[0]);

					if (!atg.getPid().equals(DUAKonstanten.ATG_LZD)) {
						datensatz.getUnscaledValue("ArtMittelwertbildung").set(1);
					}

				} catch (final ParseException ex) {
					System.err.println("Fehler beim parsing von: " + zeile[0]);
					datensatz = null;
				} catch (final ArrayIndexOutOfBoundsException ex) {
					datensatz = null;
				}
			} else {
				datensatz = null;
			}
		}

		return datensatz;
	}

	/**
	 * Erfragt die nächste Zeile innerhalb der CSV-Datei als einen Datensatz der
	 * übergebenen Attributgruppe.
	 *
	 * @param atg
	 *            eine Attributgruppe (KZD oder LZD)
	 * @param geschwDaten
	 *            definiert, ob es sich um Gescheindigkeitsdaten handelt
	 * @return ein Datensatz der übergebenen Attributgruppe mit den Daten der
	 *         nächsten Zeile oder <code>null</code>, wenn der Dateizeiger am
	 *         Ende ist
	 */
	public final Data getNaechstenAggregDatensatz(final AttributeGroup atg, final boolean geschwDaten) {
		Data datensatz = TestFSImporterGeschwKlasse.dav.createData(atg);

		if (datensatz != null) {
			final String[] zeile = this.getNaechsteZeile();
			if (zeile != null) {
				try {

					final String[] aName = new String[] { "Kfz", "PkwÄ", "KfzNk", "PkwG", "Pkw", "Krad", "Lfw", "LkwÄ",
							"PkwA", "Lkw", "Bus", "LkwK", "LkwA", "SattelKfz" };

					if (atg.getPid().equals(DUAKonstanten.ATG_LZD)) {
						datensatz = TestFSImporterGeschwKlasse.setLZDleer(datensatz, geschwDaten);
					}

					for (int j = 0; j < aName.length; j++) {

						try {
							final int wert = Integer.parseInt(zeile[j + TestFSImporterGeschwKlasse.OFFSET]);
							datensatz = TestFSImporterGeschwKlasse.setAggregAttribut(aName[j], wert, datensatz);
						} catch (final NumberFormatException e) {
							final double wert = Double
									.parseDouble(zeile[j + TestFSImporterGeschwKlasse.OFFSET].replaceAll(",", "."));
							datensatz = TestFSImporterGeschwKlasse.setAggregAttribut(aName[j], wert, datensatz);
						}
					}

					zeitStempel = df.parse(zeile[0]);

				} catch (final ParseException ex) {
					System.err.println("Fehler beim parsing von: " + zeile[0]);
					datensatz = null;
				} catch (final ArrayIndexOutOfBoundsException ex) {
					datensatz = null;
				}
			} else {
				datensatz = null;
			}
		}

		return datensatz;
	}

	/**
	 * Erfragt die nächste Zeile innerhalb der CSV-Datei als einen Datensatz der
	 * übergebenen Attributgruppe.
	 *
	 * @param atg
	 *            eine Attributgruppe (KZD oder LZD)
	 * @param geschwDaten
	 *            definiert, ob es sich um Gescheindigkeitsdaten handelt
	 * @return ein Datensatz der übergebenen Attributgruppe mit den Daten der
	 *         nächsten Zeile oder <code>null</code>, wenn der Dateizeiger am
	 *         Ende ist
	 */
	public final Data getNaechstenSpitzenStundeDatensatz(final AttributeGroup atg, final boolean geschwDaten) {
		Data datensatz = TestFSImporterGeschwKlasse.dav.createData(atg);

		if (datensatz != null) {
			final String[] zeile = this.getNaechsteZeile();
			if (zeile != null) {
				try {

					final String[] aName = new String[] { "Kfz", "PkwÄ", "KfzNk", "PkwG", "Pkw", "Krad", "Lfw", "LkwÄ",
							"PkwA", "Lkw", "Bus", "LkwK", "LkwA", "SattelKfz" };

					if (atg.getPid().equals(DUAKonstanten.ATG_LZD)) {
						datensatz = TestFSImporterGeschwKlasse.setLZDleer(datensatz, geschwDaten);
					}

					for (int j = 0; j < aName.length; j++) {

						try {
							final int wert = Integer.parseInt(zeile[j + TestFSImporterGeschwKlasse.OFFSET]);
							datensatz = TestFSImporterGeschwKlasse.setAggregAttribut("q" + aName[j], wert, datensatz);
						} catch (final NumberFormatException e) {
							final double wert = Double.parseDouble(zeile[j + TestFSImporterGeschwKlasse.OFFSET]);
							datensatz = TestFSImporterGeschwKlasse.setAggregAttribut("q" + aName[j], wert, datensatz);
						}
					}

					zeitStempel = df.parse(zeile[0]);

				} catch (final ParseException ex) {
					System.err.println("Fehler beim parsing von: " + zeile[0]);
					datensatz = null;
				} catch (final ArrayIndexOutOfBoundsException ex) {
					datensatz = null;
				}
			} else {
				datensatz = null;
			}
		}

		return datensatz;
	}

	/**
	 * Setzt einen Attribut in Datensatz.
	 *
	 * @param attributName
	 *            Name des Attributs
	 * @param destWert
	 *            Wert des Attributs
	 * @param data
	 *            der Datensatz
	 */
	private static void setAttribut(final String attributName, final long destWert, final Data data) {

		long wert = destWert;
		if (attributName.startsWith("v") && (wert >= 255)) {
			wert = -1;
		}

		try {
			DUAUtensilien.getAttributDatum(attributName + ".Wert", data).asScaledValue().set(wert);
		} catch (final UnsupportedOperationException e) {
			DUAUtensilien.getAttributDatum(attributName + ".Wert", data).asTimeValue().setSeconds(wert);
		}
		DUAUtensilien.getAttributDatum(attributName + ".Status.Erfassung.NichtErfasst", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.PlFormal.WertMax", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.PlFormal.WertMin", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.PlLogisch.WertMaxLogisch", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.PlLogisch.WertMinLogisch", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.MessWertErsetzung.Implausibel", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Status.MessWertErsetzung.Interpoliert", data).asUnscaledValue()
		.set(DUAKonstanten.NEIN);
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Index", data).asScaledValue().set(1.0);
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Verfahren", data).asUnscaledValue().set(0);
	}

	/**
	 * Setzt einen Attribut in aggregierten Datensatz.
	 *
	 * @param attributName
	 *            Name des Attributs
	 * @param setzWert
	 *            Wert des Attributs
	 * @param datensatz
	 *            der Datensatz
	 * @return der veränderte Datensatz
	 */
	private static Data setAggregAttribut(final String attributName, final long setzWert, final Data datensatz) {
		final Data data = datensatz;

		long wert = setzWert;
		if (attributName.startsWith("v") && (wert >= 255)) {
			wert = -1;
		}

		try {
			DUAUtensilien.getAttributDatum(attributName + ".Wert", data).asScaledValue().set(wert);
		} catch (final UnsupportedOperationException e) {
			DUAUtensilien.getAttributDatum(attributName + ".Wert", data).asTimeValue().setSeconds(wert);
		}
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Index", data).asScaledValue().set(1.0);
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Verfahren", data).asUnscaledValue().set(0);

		return datensatz;
	}

	/**
	 * Setzt einAttribut in aggregierten Datensatz.
	 *
	 * @param attributName
	 *            Name des Attributs
	 * @param setzWert
	 *            Wert des Attributs
	 * @param datensatz
	 *            der Datensatz
	 * @return der veränderte Datensatz
	 */
	private static Data setAggregAttribut(final String attributName, final double setzWert, final Data datensatz) {
		final Data data = datensatz;

		double wert = setzWert;
		if (attributName.startsWith("v") && (wert >= 255)) {
			wert = -1;
		}

		DUAUtensilien.getAttributDatum(attributName + ".Wert", data).asScaledValue().set(wert);
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Index", data).asScaledValue().set(1.0);
		DUAUtensilien.getAttributDatum(attributName + ".Güte.Verfahren", data).asUnscaledValue().set(0);

		return datensatz;
	}

	/**
	 * Setzt einen LZDatensatz als leer.
	 *
	 * @param datensatz
	 *            Der Datensatz
	 * @param geschwDaten
	 *            true, wenn auch Geschwindigkeitsklassendaten erzeugt werden
	 *            sollen
	 * @return der geaenderte Datensatz
	 */
	public static final Data setLZDleer(final Data datensatz, final boolean geschwDaten) {
		final String postfix = new String("GeschwKlasse");
		final String[] praefix = new String[] { "q", "v", "s", "v85" };
		final String[] aName = new String[] { "Kfz", "PkwÄ", "KfzNk", "PkwG", "Pkw", "Krad", "Lfw", "LkwÄ", "PkwA",
				"Lkw", "Bus", "LkwK", "LkwA", "SattelKfz" };

		for (final String element : praefix) {
			for (final String element2 : aName) {
				TestFSImporterGeschwKlasse.setAttribut(element + element2, -1, datensatz);
			}
		}

		if (!geschwDaten) {
			return datensatz;
		}

		for (final String element : aName) {
			final String attribut = praefix[0] + element + postfix;
			final Array arrayField = datensatz.getItem(attribut).asArray();
			arrayField.setLength(16);
			for (int i = 0; i < 16; i++) {
				TestFSImporterGeschwKlasse.setAttribut(attribut + "." + String.valueOf(i), -1, datensatz);
			}
		}
		return datensatz;
	}
}
